@@ -32,7 +32,7 @@ On a Debian/Ubuntu system you can install these modules using apt:
     sys.exit(1)
 
 from qtpy import QtWidgets
-from qtpy.QtCore import Qt
+from qtpy.QtCore import Qt, Signal
 
 try:
     # Qt 5.12 / PyQt 5.13 is unable to use QtWebEngineWidgets unless it is
@@ -52,6 +52,7 @@ from .widgets import standard
 from .widgets import startup
 from .settings import Session
 from .settings import Settings
+from . import cmd
 from . import cmds
 from . import core
 from . import compat
@@ -322,7 +323,7 @@ def restore_session(args):
         args.repo = session.repo
 
 
-def application_init(args, update=False, app_name='Git Cola'):
+def application_init(args, update=False, app_name='Git Cola', setup_worktree=True):
     """Parses the command-line arguments and starts git-cola"""
     # Ensure that we're working in a valid git repository.
     # If not, try to find one.  When found, chdir there.
@@ -333,9 +334,10 @@ def application_init(args, update=False, app_name='Git Cola'):
     timer = context.timer
     timer.start('init')
 
-    new_worktree(context, args.repo, args.prompt)
-    if update:
-        context.model.update_status()
+    if setup_worktree:
+        new_worktree(context, args.repo, args.prompt)
+        if update:
+            context.model.update_status()
 
     timer.stop('init')
     if args.perf:
@@ -346,6 +348,7 @@ def application_init(args, update=False, app_name='Git Cola'):
 def new_context(args, app_name='Git Cola'):
     """Create top-level ApplicationContext objects"""
     context = ApplicationContext(args)
+    context.timestamp = time.time()
     context.settings = args.settings or Settings.read()
     context.git = git.create()
     context.cfg = gitcfg.create(context)
@@ -615,9 +618,11 @@ class ApplicationContext:
     def __init__(self, args):
         self.args = args
         self.app = None  # ColaApplication
+        self.command_bus = None  # cmd.CommandBus
         self.git = None  # git.Git
         self.cfg = None  # gitcfg.GitConfig
         self.model = None  # main.MainModel
+        self.notifier = Notifier(self)
         self.timer = None  # Timer
         self.runtask = None  # qtutils.RunTask
         self.settings = None  # settings.Settings
@@ -629,9 +634,35 @@ class ApplicationContext:
     def set_view(self, view):
         """Initialize view-specific members"""
         self.view = view
+        self.notifier.setParent(view)
+        self.command_bus = cmd.CommandBus(parent=view)
         self.runtask = qtutils.RunTask(parent=view)
 
 
+class Notifier(QtCore.QObject):
+    """Message bus for generic one-off notifications"""
+
+    message = Signal(object)
+
+    def __init__(self, context, parent=None):
+        super().__init__(parent)
+        self.context = context
+
+    def notify(self, message):
+        """Send messages to listeners"""
+        self.message.emit(message)
+
+    def listen(self, message, callback):
+        """Subscribe a callback specific messages"""
+
+        def listener(current_message, message=message, callback=callback):
+            """Fire a callback when specific messages are seen"""
+            if current_message is message:
+                callback()
+
+        self.message.connect(listener, type=Qt.QueuedConnection)
+
+
 def find_git():
     """Return the path of git.exe, or None if we can't find it."""
     if not utils.is_win32():
